require 'middleclass'
require 'sprites'
require 'particles'
require 'sounds'

--shapes categories

BOMB = 1
EXPLOSION = 2
DESTRUCTIBLE = 3
OTHER = 16

Bomb = class('Bomb')

SmallBomb = class('SmallBomb',Bomb)
function SmallBomb:initialize(x,y)
	self.body = love.physics.newBody(world, x,y)
	self.shape = love.physics.newCircleShape(self.body,0,0,16)
	--self.body:setLinearDamping(1)
	--self.body:setAngularDamping(1)
	self.body:setMassFromShapes()
	self.shape:setData(self)
	self.shape:setCategory(BOMB)
	--self.shape:setMask(BOMB)
    self.detonated = false
	self.timer0 = 0.02
    self.timer = self.timer0
	self.removed = false
end

function SmallBomb:remove()
	--self.shape:setMask(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16)
    --print('smallbomb remove')
	self.shape:destroy()
	self.body:destroy()
	self.removed = true
	compact_counter = compact_counter + 1
end

function SmallBomb:draw()
	if not self.removed then
		local x,y,a
		x,y = self.body:getPosition()
		a = self.body:getAngle()
		--local color = self.timer / self.timer0
		--love.graphics.setColor(255*color,255*color,255*color)
		love.graphics.draw(sprites.bomb,x,y,a,1,1,16,16)
	end
end

function SmallBomb:update(dt)
	if self.removed then return end
	local x,y,vx,vy
	x,y = self.body:getPosition()
	vx,vy = self.body:getLinearVelocity()
	if x < 16 then self.body:setPosition(W1, y) end
	if x > W1 then self.body:setPosition(16, y) end
	if y < 16 then self.body:setPosition(x, H1) end
	if y > H1 then self.body:setPosition(x, 16) end
    if self.detonated then self.timer = self.timer - dt
        if self.timer <= 0 then
            self:remove()
            table.insert(objects, SmallExplosion(x,y,vx,vy))
        end
    end
end

function SmallBomb:detonate()
	if not self.detonated then
		self.detonated = true
		chain = chain + 1
		--reset global chain timer
		chain_timer = 1
	end
end

function SmallBomb:cb_collide(object, contact)
	--print('smallbomb collide with',object)
	--if instanceOf(Explosion, object) then
		--local x1,y1 = self.body:getPosition()
		--local x2,y2 = object.body:getPosition()
		--local dx = x1-x2
		--local dy = y1-y2
		--dist = math.sqrt(dx^2+dy^2)
		--if dist < 64 then self:detonate() end
		----self:detonate()
		--self.body:applyImpulse(dx/10,dy/10)
	--else
		local vx,vy = contact:getVelocity()
		local v = math.sqrt(vx^2+vy^2)
		--print(v)
		if v > 500 then
			self:detonate()
			--object:detonate()
		end
	--end
end

Explosion = class('Explosion')

SmallExplosion = class('SmallExplosion',Explosion)
function SmallExplosion:initialize(x,y,vx,vy)
	--print('create explosion at',x,y)
	--if x < 0 or x > W or y < 0 or y > W then return end
	self.shockwave_disabled = false
	self.removed = false
	self.shockwave = particles_shockwave(sprites.shockwave)
	self.shockwave:setPosition(x,y)
	self.explosion = particles_explosion(sprites.explosion)
	self.explosion:setPosition(x,y)
	self.body = love.physics.newBody(world,x,y,1)
	self.shape = love.physics.newCircleShape(self.body, 0,0, 96)
	self.shape:setCategory(EXPLOSION)
	self.shape:setMask(EXPLOSION)
	self.shape:setSensor(true)
	self.shape:setData(self)
	self.sound = love.audio.newSource(sounds.explosion)
	love.audio.play(self.sound)
	self.body:setLinearVelocity(vx or 0, vy or 0)
	self.shockwave:start()
	self.explosion:start()
	--print('explosion created')
end

function SmallExplosion:remove()
	--self.shape:setMask(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16)
	--print('remove explosion')
	self.removed = true
	self.shape:destroy()
	self.body:destroy()
	compact_counter = compact_counter + 1
    --print('explosion removed')
end

function SmallExplosion:draw()
	if not self.removed then
		love.graphics.setBlendMode("alpha")
		love.graphics.draw(self.shockwave,0,0)
		love.graphics.setBlendMode("additive")
		love.graphics.draw(self.explosion,0,0)
	end
end

function SmallExplosion:update(dt)
	if self.removed then return end
	local x,y
	x,y = self.body:getPosition()
	if not self.shockwave:isEmpty() or self.shockwave:isActive() then
		self.shockwave:setPosition(x,y)
		self.shockwave:update(dt)
	elseif not self.shockwave_disabled then
		self.shape:setMask(BOMB,EXPLOSION)
		self.shockwave_disabled = true
	end
	if not self.explosion:isEmpty() or self.explosion:isActive() then
		self.explosion:setPosition(x,y)
		self.explosion:update(dt)
	else
		self:remove()
	end
end

function SmallExplosion:cb_collide(object, contact)
	if instanceOf(Bomb, object) then
		local x1,y1 = self.body:getPosition()
		local x2,y2 = object.body:getPosition()
		local dx = x2-x1
		local dy = y2-y1
		dist = math.sqrt(dx^2+dy^2)
		if dist < 64 then object:detonate() end
		--self:detonate()
		object.body:applyImpulse(dx/10,dy/10)
	end
end

Crate = class('Crate')
function Crate:initialize(x,y)
	self.health = 1
	self.body = love.physics.newBody(world, x,y)
	self.shape = love.physics.newRectangleShape(self.body, 0,0, 32,32)
	self.shape:setData(self)
	self.body:setMassFromShapes()
end

function Crate:remove()
	self.removed = true
	self.shape:destroy()
	self.body:destroy()
	compact_counter = compact_counter + 1
end

function Crate:update(dt)
	if not self.removed then
		if self.health <= 0 then self:remove() end
	end
end

function Crate:draw()
	if not self.removed then
		local x,y = self.body:getPosition()
		local a = self.body:getAngle()
		--love.graphics.setColor(255,255,255,255*self.health)
		love.graphics.draw(sprites.crate, x,y, a, 1,1, 16,16)
		--love.graphics.polygon("fill", self.shape:getPoints())
	end
end

function Crate:cb_collide(object, contact)
	if instanceOf(Explosion, object) then
		local x1,y1 = self.body:getPosition()
		local x2,y2 = object.body:getPosition()
		local dx = x1-x2
		local dy = y1-y2
		self.body:applyImpulse(dx/10,dy/10)
		self.health = self.health - 0.4
	end
end
