require 'entity'
require 'util'

DEBUG = false
PAUSED = false

W = love.graphics.getWidth()
H = love.graphics.getHeight()
W1 = W - 16
H1 = H - 16

level = Gamestate.new()

function level:enter(from,name)
	slomo_factor = 1
	compact_counter = 0
	chain = 0
	chain_timer = 0
	max_chain = 0
	chain_lock = false
	world = love.physics.newWorld(0,0,W,H)
	world:setCallbacks(cb_collide,nil,nil,nil)
	objects = {}
	for i=1,100 do
		table.insert(objects, SmallBomb(math.random(love.graphics.getWidth()), math.random(love.graphics.getHeight())))
	end
	--collectgarbage("stop")
end

function level:leave()
	objects = compact(objects)
	for i,o in ipairs(objects) do o:remove() end
	objects = nil
	world = nil
	collectgarbage("collect")
end

function level:draw()
	love.graphics.setColor(255,255,255)
	love.graphics.setBlendMode("alpha")
    love.graphics.drawq(sprites.background, quads.background, 0,0)
	for i,o in ipairs(objects) do
		o:draw()
	end
	if DEBUG then
		love.graphics.print(memory, 20, 20)
		love.graphics.print(bodies, 20, 40)
	else
		love.graphics.print(max_chain, 20, 50)
	end
end

function level:update(dt)
	if not PAUSED then
		dt = dt / slomo_factor
		world:update(dt)
		for i,o in ipairs(objects) do
			o:update(dt)
		end
		if chain_timer > 0 then
			chain_timer = chain_timer - dt
		else
			last_chain,chain = chain,0
			if max_chain < last_chain then max_chain = last_chain end
			--if last_chain ~= 0 then print(last_chain,max_chain) end
			chain_lock = false
		end
		if compact_counter >= 50 then
			objects = compact(objects)
			compact_counter = 0
		end
	end
	if DEBUG then
		memory = collectgarbage("count")
		bodies = world:getBodyCount()
	end
end

function level:mousepressed( x, y, button )
	if not PAUSED then
		if button == 'l' and not chain_lock then
			table.insert(objects,SmallExplosion(x,y))
			chain_lock = true
		end
		if button == 'r' then
			table.insert(objects,SmallBomb(x,y))
			--objects[#objects]:detonate()
		end
		if button == 'm' then
			table.insert(objects,Crate(x,y))
		end
	end
end

function level:keypressed(key)
	if key == "escape" then love.event.push("q") end
	if key == "r" then Gamestate.switch(level) end
	if key == "p" then
		PAUSED = not PAUSED
		if PAUSED then love.audio.pause() else love.audio.resume() end
	end
	if key == "d" then DEBUG = not DEBUG end
	if key == "1" then slomo_factor = 1 end
	if key == "2" then slomo_factor = 2 end
	if key == "3" then slomo_factor = 3 end
	if key == "4" then slomo_factor = 4 end
	if key == "5" then slomo_factor = 5 end
end
